from qronos_django.models import QRonosImportLog
from qronos_django.settings import QRONOS_LOGGING, QRONOS_ID_LOGGING, QRONOS_DATA_LOGGING
from qronos_django.tasks import process_import


def tracker_import(tracker_id, tracker_importer_id, unique_columns, data):
    import_type = QRonosImportLog.QRonosImportType.TRACKER
    if QRONOS_LOGGING:
        import_log = QRonosImportLog.objects.create(
            import_type=import_type,
            id_log=f"Tracker ID: {tracker_id}, Tracker Importer ID: {tracker_importer_id}" if QRONOS_ID_LOGGING else "LOGGING DISABLED",
            data_log=f"Unique Columns:\n{unique_columns}\n\nData:\n{data}" if QRONOS_DATA_LOGGING else "LOGGING DISABLED",
        )
        import_log_id = import_log.id
    else:
        import_log = None
        import_log_id = None
    process_import.delay(import_log_id, import_type, tracker_id, tracker_importer_id, unique_columns, data)
    return import_log


def stage_import(stage_id, stage_importer_id, data):
    import_type = QRonosImportLog.QRonosImportType.STAGE
    if QRONOS_LOGGING:
        import_log = QRonosImportLog.objects.create(
            import_type=import_type,
            id_log=f"Stage ID: {stage_id}, Stage Importer ID: {stage_importer_id}" if QRONOS_ID_LOGGING else "LOGGING DISABLED",
            data_log=f"Data:\n{data}" if QRONOS_DATA_LOGGING else "LOGGING DISABLED",
        )
        import_log_id = import_log.id
    else:
        import_log = None
        import_log_id = None
    process_import.delay(import_log_id, import_type, stage_id, stage_importer_id, data)
    return import_log



from celery import shared_task
from qronos import QRonosError

from qronos_django.connection import get_qronos_client
from qronos_django.models import QRonosImportLog
from qronos_django.settings import QRONOS_LOGGING


@shared_task
def process_import(import_log_id, import_type, *import_args):
    try:
        qronos = get_qronos_client()
        if import_type == QRonosImportLog.QRonosImportType.TRACKER:
            job_id = qronos.tracker_import(*import_args)
        elif import_type == QRonosImportLog.QRonosImportType.STAGE:
            job_id = qronos.stage_import(*import_args)
        else:
            raise QRonosError("Invalid Import Type")
    except QRonosError as err:
        qronos_log_data = {
            "job_id": None,
            "error_log": repr(err),
            "status": "FAIL",
        }
    else:
        qronos_log_data = {
            "job_id": job_id,
            "status": "UNKNOWN",
            "error_log": "",
        }
    if QRONOS_LOGGING:
        QRonosImportLog.objects.filter(id=import_log_id).update(**qronos_log_data)


@shared_task
def update_qronos_log_status(log_ids):
    qronos = get_qronos_client()
    for import_log in QRonosImportLog.objects.filter(id__in=log_ids).exclude(job_id=None):
        import_log.update_import_status(qronos)



from django.contrib import admin, messages

from qronos_django.models import QRonosImportLog
from qronos_django.tasks import update_qronos_log_status


class QRonosImportLogAdmin(admin.ModelAdmin):
    list_display = ['id', 'job_id', 'sent', 'status', 'status_updated']
    actions = ["update_status"]

    def update_status(self, request, queryset):
        update_qronos_log_status.delay([qi.id for qi in queryset.exclude(job_id=None)])
        self.message_user(request, "Statuses sent for update, please refresh shortly.", messages.SUCCESS)

    update_status.short_description = "Update Import Status"


admin.site.register(QRonosImportLog, QRonosImportLogAdmin)
